; TEST4DV.ASM
;   This correction for a program DESQTEST.COM that
;   was uploaded by Marc Rubin as DESQTEST.ARC.
;   He had a small error in the program listing
;   which effectively over wrote the version number
;   with ZERO and always reported DESQview not present.
;
;   Description:  Check for the presence of DESQview using DOS set system date.
;                 Set the errorlevel on exit for use in batch files.
;
;   DESQview monitors many DOS and BIOS calls and performs its own internal
;   routines for some of them.  When using the DOS 'Set System Date' call,
;   DESQview will place the version number in BX when CX,DX contains a value
;   of 'DESQ'.  The DOS call does not modify BX so this is a simply method
;   for determining if DESQview is present.  Since 'DESQ' is not a valid
;   date, the system date will not be affected.  Keep in mind that all we
;   really need is the lower byte of BX.  This is important if we want to
;   use this call to set the DOS errorlevel for use in a batch file.
;
;   The Progam: Compare the following notes to the program listing.
;
;   1: Save the registers we will modify.
;   2: Set AX for the 'Set System Date' DOS call.
;   3: load CX,DX with the value 'DESQ'.
;   4: zero out BX in case DESQview is not present.
;   5: call DOS.
;
;   6: Copy BX into AX, keep in mind that all we really need is BL.
;      AL will be set to the version number if DESQview is
;      present or it will be zero.
;   7: restore the other registers we used.
;   8: Load AH with the DOS 'Terminate with Return Code' call to set the
;      errorlevel for the batch file.
;   9: End the program using DOS INT 21h.
;
;   Assembling the program: This listing can be assembled and linked as is
;                           by following the steps below.
;
;   MASM TEST4DV;
;   LINK TEST4DV;
;   EXE2BIN TEST4DV.EXE TEST4DV.COM
;   ERASE TEST4DV.EXE
;
; Using TEST4DV.COM
;
;   Place TEST4DV in your batch file.  If DESQview is preset, then
;   the errorlevel will contain the version number of DESQview.
;   If it is not, then the errorlevel will be set to ZERO.
;

CODE      SEGMENT
          ASSUME CS:CODE,DS:CODE,ES:CODE,SS:CODE
          ORG    100H

ENTRY:    PUSH   BX               ; 1:  save the registers
          PUSH   CX               ;       that are used,
          PUSH   DX               ;       AX is destroyed
          MOV    AX,2B01H         ; 2:  DOS set date
          MOV    CX,'DE'          ; 3:  invalid date for
          MOV    DX,'SQ'          ;       DOS call
          XOR    BX,BX            ; 4:  clear, DESQview will set if present
                                  ;      otherwise it will remain 0.
          INT    21H              ; 5:  call DOS
          MOV    AX,BX            ; 6:  get the returned value.
                                  ;     AL contains the version number or 0.
          POP    DX               ; 7:  restore saved
          POP    CX               ;      registers
          POP    BX
          MOV    AH, 04CH         ; 8:  DOS terminate with return code in AL
          INT    21H              ; 9:  all done, exit.
CODE      ENDS
          END    ENTRY
